﻿unit getosinf;

////////////////////////////////////////////////////////////////
//
// OSのversionやら32bit/64bitの区別やらを行うツール類
//
// Copyright (C) 2009-2014 Ayakawa,Marinosuke & MarleySoft
//
// http://homepage1.nifty.com/Ayakawa/
//
// rev  0
// date 2009/12/03
// auth 綾川鞠乃介
// memo 公開用にちょっとまとめ
//
// rev  1
// date 2014/10/06
// auth 綾川鞠乃介
// memo GetVersionExの非推奨化対策
//
// rev  2
// date 2015/08/19
// auth 綾川鞠乃介
// memo win7,win8,win8.1,win10
//
// rev  3
// date 2016/08/04
// auth 綾川鞠乃介
// memo win10 anniversary update
//
////////////////////////////////////////////////////////////////

interface

uses
  Winapi.Windows,
  System.SysUtils;

const
  // for GetSystemInfo/GetNativeSystemInfo
  PROCESSOR_ARCHITECTURE_AMD64   = 9;     // x64 (AMD or Intel)
  PROCESSOR_ARCHITECTURE_IA64    = 6;     // Intel Itanium Processor Family (IPF)
  PROCESSOR_ARCHITECTURE_INTEL   = 0;     // x86
  PROCESSOR_ARCHITECTURE_UNKNOWN = $ffff; // Unknown processor.

type
  EGetOSVerInfo = class(Exception);

function GetMajorVer:DWORD;
function GetMinorVer:DWORD;
function GetBuildNumber:DWORD;
function GetOS:DWORD;
function GetCSDVersion:string;
function GetSPMajorVer:WORD;
function GetSPMinorVer:WORD;


// WOW64上で稼働しているなら真
function IsWow64Process(hProcess:THandle; var Wow64Process:BOOL):BOOL; stdcall;
function IsWow64:boolean;

// OSがx86_64なら真
function IsX64:boolean;
// OSがx86なら真
function IsX86:boolean;

procedure GetNativeSystemInfo(var info:TSystemInfo); stdcall;

// WoW64制御 - delayedは使わないように。
function Wow64DisableWow64FsRedirection(var OldValue:pointer):BOOL; stdcall;
function Wow64RevertWow64FsRedirection(OldValue:pointer):BOOL; stdcall;
function Wow64EnableWow64FsRedirection(Wow64FsEnableRedirection:BOOL):BOOL; stdcall;

//
function IsNT351:boolean;
function IsW2K:boolean;
function IsXP:boolean;
function IsVista:boolean;
function IsWin7:boolean;
function IsWin8:boolean;
function IsWin81:boolean;
function IsWin10:boolean;
function IsWIn11:boolean;

// 古いプログラムの互換用
function IsVistaOrLater:boolean;
function IsWin7OrLater:boolean;
function IsWin8OrLater:boolean;
function IsWin81OrLater:boolean;
function IsWin10OrLater:boolean;
function IsWin10AUOrLater:boolean;
function IsWin11OrLater:boolean;


// kernel ver.
type
  TRTL_OSVERSIONINFOEXW = TOSVERSIONINFOEXW;
  PRTL_OSVERSIONINFOEXW = ^TRTL_OSVERSIONINFOEXW;

function RtlGetVersion(var info:TRTL_OSVERSIONINFOEXW):boolean;

implementation

var
  ok      : boolean;
  verinfo : TOSVERSIONINFOEX;
  faddinfo : string;

type
  NTSTATUS = LONG;

const
  STATUS_SUCCESS = 0;

function GetMajorVer;
begin
  if NOT ok then raise EGetOSVerInfo.Create('ERR : GetMajorVer');
  Result:=verinfo.dwMajorVersion;
end;

function GetMinorVer;
begin
  if NOT ok then raise EGetOSVerInfo.Create('ERR : GetMinorVer');
  Result:=verinfo.dwMinorVersion;
end;

function GetBuildNumber;
begin
  if NOT ok then raise EGetOSVerInfo.Create('ERR : GetBuildNumber');
  Result:=verinfo.dwBuildNumber;
end;

function GetOS;
begin
  if NOT ok then raise EGetOSVerInfo.Create('ERR : GetOS');
  Result:=verinfo.dwPlatformId;
end;

function  GetCSDVersion:string;
begin
  if NOT ok then raise EGetOSVerInfo.Create('ERR : GetCSDVersion');
  result:=faddinfo;
end;

function GetSPMajorVer:WORD;
begin
  if NOT ok then raise EGetOSVerInfo.Create('ERR : GetSPMajorVer');
  result:=verinfo.wServicePackMajor;
end;

function GetSPMinorVer:WORD;
begin
  if NOT ok then raise EGetOSVerInfo.Create('ERR : GetSPMinorVer');
  result:=verinfo.wServicePackMinor
end;

function    IsNT351:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            (GetMajorVer=3) AND
            (GetMinorVer=51);
  except
    ON EGetOSVerInfo DO ;
  end;
end;

function IsW2K:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            (GetMajorVer=5) AND
            (GetMinorVer=0);
  except
    ON EGetOSVerInfo DO ;
  end;
end;

function IsXP:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            (GetMajorVer=5) AND
            (GetMinorVer=1);
  except
    ON EGetOSVerInfo DO ;
  end;
end;

function IsVista:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            (GetMajorVer=6) AND
            (GetMinorVer=0);
  except
    ON EGetOsVerInfo DO ;
  end;
end;

function IsWin7:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            (GetMajorVer=6) AND
            (GetMinorVer=1);
  except
    ON EGetOsVerInfo DO ;
  end;
end;

function IsWin8:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            (GetMajorVer=6) AND
            (GetMinorVer=2);
  except
    ON EGetOsVerInfo DO ;
  end;
end;

function IsWin81:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            (GetMajorVer=6) AND
            (GetMinorVer=3);
  except
    ON EGetOsVerInfo DO ;
  end;
end;

function IsWin10:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            (GetMajorVer=10) AND
            (GetMinorVer=0);
  except
    ON EGetOsVerInfo DO ;
  end;
end;

function IsWin11:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            (GetMajorVer=10) AND
            (GetMinorVer=0) AND
            (GetBuildNumber=22000);
  except
    ON EGetOsVerInfo DO ;
  end;
end;

function IsVistaOrLater:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND (GetMajorVer>=6);
  except
    ON EGetOSVerInfo DO ;
  end;
end;

function IsWin7OrLater:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            ((GetMajorVer>6) OR ((GetMajorVer=6) AND (GetMinorVer>0)));
  except
    ON EGetOSVerInfo DO ;
  end;
end;

function IsWin8OrLater:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            ((GetMajorVer>6) OR ((GetMajorVer=6) AND (GetMinorVer>1)));
  except
    ON EGetOSVerInfo DO ;
  end;
end;

function IsWin81OrLater:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            ((GetMajorVer>6) OR ((GetMajorVer=6) AND (GetMinorVer>2)));
  except
    ON EGetOSVerInfo DO ;
  end;
end;

function IsWin10OrLater:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND (GetMajorVer>=10);
  except
    ON EGetOSVerInfo DO ;
  end;
end;

function IsWin10AUOrLater:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            ( (GetMajorVer>10) OR
              ((GetMajorVer=10) AND
               ((GetMinorVer>0) OR (GetBuildNumber>=14393)))
            );
  except
    ON EGetOSVerInfo DO ;
  end;
end;

function IsWin11OrLater:boolean;
begin
  result:=false;
  try
    result:=(GetOS=VER_PLATFORM_WIN32_NT) AND
            ( (GetMajorVer>10) OR
              ((GetMajorVer=10) AND
               ((GetMinorVer>0) OR
                (GetMinorVer=0) AND (GetBuildNumber>=22000)
               )
              )
            );
  except
    ON EGetOSVerInfo DO ;
  end;
end;

var
  _IsWow64Process:function (hProcess:THandle; var Wow64Process:BOOL):BOOL; stdcall;
  _GetNativeSystemInfo:procedure (var info:TSystemInfo); stdcall;
  _Wow64DisableWow64FsRedirection:function(var OldValue:pointer):BOOL; stdcall;
  _Wow64RevertWow64FsRedirection:function(OldValue:pointer):BOOL; stdcall;
  _Wow64EnableWow64FsRedirection:function(Wow64FsEnableRedirection:BOOL):BOOL; stdcall;
  _RtlGetVersion:function(var info:TRTL_OSVERSIONINFOEXW):NTSTATUS; stdcall;
  HKernel32,HNtDll : THandle;

function IsWow64Process(hProcess:THandle; var Wow64Process:BOOL):BOOL; stdcall;
begin
  if @_IsWow64Process<>nil then
    result:=_IsWow64Process(hProcess,Wow64Process)
  else begin
    Wow64Process:=false;
    result:=false;
    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
  end;
end;

function Wow64DisableWow64FsRedirection(var OldValue:pointer):BOOL; stdcall;
begin
  if @_Wow64DisableWow64FsRedirection<>nil then
    result:=_Wow64DisableWow64FsRedirection(OldValue)
  else begin
    result:=false;
    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
  end;
end;

function Wow64RevertWow64FsRedirection(OldValue:pointer):BOOL; stdcall;
begin
  if @_Wow64RevertWow64FsRedirection<>nil then
    result:=_Wow64RevertWow64FsRedirection(OldValue)
  else begin
    result:=false;
    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
  end;
end;

function Wow64EnableWow64FsRedirection(Wow64FsEnableRedirection:BOOL):BOOL; stdcall;
begin
  if @_Wow64EnableWow64FsRedirection<>nil then
    result:=_Wow64EnableWow64FsRedirection(Wow64FsEnableRedirection)
  else begin
    result:=false;
    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
  end;
end;

procedure GetNativeSystemInfo(var info:TSystemInfo); stdcall;
begin
  if @_GetNativeSystemInfo<>nil then
    _GetNativeSystemInfo(info)
  else
    GetSystemInfo(info);
end;

function IsWow64:boolean;
var
  b : BOOL;
begin
  result:=IsWow64Process(GetCurrentProcess,b);
  if result then result:=b;
end;

function IsX64:boolean;
var
  info : TSystemInfo;
begin
  GetNativeSystemInfo(info);
  result:=(info.wProcessorArchitecture=PROCESSOR_ARCHITECTURE_AMD64);
end;

function IsX86:boolean;
var
  info : TSystemInfo;
begin
  GetNativeSystemInfo(info);
  result:=(info.wProcessorArchitecture=PROCESSOR_ARCHITECTURE_INTEL);
end;

function RtlGetVersion(var info:TRTL_OSVERSIONINFOEXW):boolean;
begin
  if @_RtlGetVersion=nil then
    result:=false
  else
    result:=(STATUS_SUCCESS=_RtlGetVersion(info));
end;

procedure Init;
var
  sv : integer;
  p  : TFarProc;
  info : TRTL_OSVERSIONINFOEXW;
begin
  verinfo.dwOSVersionInfoSize := sizeof(TOSVERSIONINFO);
  ok:=GetVersionEx(verinfo);
  IF ok THEN
    faddinfo:=string(verinfo.szCSDVersion)
  ELSE
    faddinfo:='';
  //
  _IsWow64Process:=nil;
  _GetNativeSystemInfo:=nil;
  _Wow64DisableWow64FsRedirection:=nil;
  _Wow64RevertWow64FsRedirection:=nil;
  _Wow64EnableWow64FsRedirection:=nil;
  _RtlGetVersion:=nil;
  sv:=SetErrorMode(SEM_FAILCRITICALERRORS);
  try
    HKernel32:=LoadLibrary('kernel32.dll');
    IF HKernel32<>0 THEN BEGIN
      p:=GetProcAddress(HKernel32,'IsWow64Process');
      IF p<>nil THEN BEGIN
        @_IsWow64Process:=p;
      END;
      p:=GetProcAddress(HKernel32,'GetNativeSystemInfo');
      IF p<>nil THEN BEGIN
        @_GetNativeSystemInfo:=p;
      END;
      p:=GetProcAddress(HKernel32,'Wow64DisableWow64FsRedirection');
      if p<>nil then begin
        @_Wow64DisableWow64FsRedirection:=p;
      end;
      p:=GetProcAddress(HKernel32,'Wow64RevertWow64FsRedirection');
      if p<>nil then begin
        @_Wow64RevertWow64FsRedirection:=p;
      end;
      p:=GetProcAddress(HKernel32,'Wow64EnableWow64FsRedirection');
      if p<>nil then begin
        @_Wow64EnableWow64FsRedirection:=p;
      end;
    END;
    //
    HNtDll:=LoadLibrary('ntdll.dll');
    if HNtDll<>0 then begin
      p:=GetProcAddress(HNtDll,'RtlGetVersion');
      if p<>nil then begin
        @_RtlGetVersion:=p;
        // kernel ver.で再取得
        info.dwOSVersionInfoSize:=sizeof(info);
        if RtlGetVersion(info) then verinfo:=info; // kernel ver.で情報を上書き
      end;
    end;
  finally
    SetErrorMode(sv);
  end;
end;

procedure Done;
begin
  if HKernel32<>0 then begin
    _IsWow64Process:=nil;
    _GetNativeSystemInfo:=nil;
    _Wow64DisableWow64FsRedirection:=nil;
    _Wow64RevertWow64FsRedirection:=nil;
    _Wow64EnableWow64FsRedirection:=nil;
    FreeLibrary(HKernel32);
    Hkernel32:=0;
  end;
  if HNtDll<>0 then begin
    _RtlGetVersion:=nil;
    FreeLibrary(HNtDll);
    HNtDll:=0;
  end;
end;

initialization
  init;

finalization
  done;


end.
