﻿unit HighDPIAPIs;

{**********************************************************************
 *
 * High DPI APIs / Windows 8.1 or Later
 *
 **********************************************************************}

interface

uses
  Winapi.Windows,
  Winapi.Messages,
  Winapi.Multimon,
  System.SysUtils;

const
  // MONITOR_DPI_TYPE
  MDT_Effective_DPI  = 0;
  MDT_Angular_DPI    = 1;
  MDT_Raw_DPI        = 2;
  MDT_Default        = MDT_Effective_DPI;

  // PROCESS_DPI_AWARENESS
  Process_DPI_Unaware            = 0;
  Process_System_DPI_Aware       = 1;
  Process_Per_Monitor_DPI_Aware  = 2;

  // SHELL_UI_COMPONENT
  SHELL_UI_COMPONENT_TASKBARS          = 0;
  SHELL_UI_COMPONENT_NOTIFICATIONAREA  = 1;
  SHELL_UI_COMPONENT_DESKBAND          = 2;

  // message
  WM_DPICHANGED = $02E0;

type
  TWMDPIChanged = record
    Msg: Cardinal;
    WPARAMFiller : TDWordFiller;
    YDPI : WORD;
    XDPI : WORD;
    pRect: LPARAM; // pointer to a RECT...?
    Result: LRESULT;
  end;

function GetDPIForMonitor(hm:HMONITOR; dpiType:Int32; var spiX,spiY:UINT):HRESULT; stdcall;
function GetDPIForShellUiComponent(component:NativeInt):UINT; stdcall;
function GetProcessDPIAwareness(hp:THandle; var value:NativeInt):HRESULT; stdcall;
function SetProcessDPIAwareness(value:NativeInt):HRESULT; stdcall;
function LogicalToPhysicalPointForPerMonitorDPI(hw:HWND; lpPoint:PPoint):BOOL; stdcall;
function PhysicalToLogicalPointForPerMonitorDPI(hw:HWND; lpPoint:PPoint):BOOL; stdcall;

implementation

uses
  Getosinf;

var
  _GetDPIForMonitor: function(hm:HMONITOR; dpiType:NativeInt; var spiX,spiY:UINT):HRESULT; stdcall;
  _GetDPIForShellUiComponent : function(component:NativeInt):UINT; stdcall;
  _GetProcessDPIAwareness : function(hp:THandle; var value:NativeInt):HRESULT; stdcall;
  _SetProcessDPIAwareness : function(value:NativeInt):HRESULT; stdcall;
  _LogicalToPhysicalPointForPerMonitorDPI : function(hw:HWND; lpPoint:PPoint):BOOL; stdcall;
  _PhysicalToLogicalPointForPerMonitorDPI : function(hw:HWND; lpPoint:PPoint):BOOL; stdcall;
  Hshcore:THandle;

function FaultGetDPIForMonitor(hm:HMONITOR; dpiType:NativeInt; var spiX,spiY:UINT):HRESULT; stdcall;
begin
  result:=E_INVALIDARG;
  SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
end;

function FaultGetDPIForShellUiComponent(component:NativeInt):UINT; stdcall;
begin
  result:=0;
  SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
end;

function FaultGetProcessDPIAwareness(hp:THandle; var value:NativeInt):HRESULT; stdcall;
begin
  result:=E_INVALIDARG;
  SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
end;

function FaultSetProcessDPIAwareness(value:NativeInt):HRESULT; stdcall;
begin
  result:=E_INVALIDARG;
  SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
end;

function FaultLogicalToPhysicalPointForPerMonitorDPI(hw:HWND; lpPoint:PPoint):BOOL; stdcall;
begin
  result:=FALSE;
  SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
end;

function FaultPhysicalToLogicalPointForPerMonitorDPI(hw:HWND; lpPoint:PPoint):BOOL; stdcall;
begin
  result:=FALSE;
  SetLastError(ERROR_CALL_NOT_IMPLEMENTED);
end;

function GetDPIForMonitor(hm:HMONITOR; dpiType:Int32; var spiX,spiY:UINT):HRESULT; stdcall;
begin
  result:=_GetDPIForMonitor(hm,dpiType,spiX,spiY);
end;

function GetDPIForShellUiComponent(component:NativeInt):UINT; stdcall;
begin
  result:=_GetDPIForShellUiComponent(component);
end;

function GetProcessDPIAwareness(hp:THandle; var value:NativeInt):HRESULT; stdcall;
begin
  result:=_GetProcessDPIAwareness(hp,value);
end;

function SetProcessDPIAwareness(value:NativeInt):HRESULT; stdcall;
begin
  result:=_SetProcessDPIAwareness(value);
end;

function LogicalToPhysicalPointForPerMonitorDPI(hw:HWND; lpPoint:PPoint):BOOL; stdcall;
begin
  result:=_LogicalToPhysicalPointForPerMonitorDPI(hw,lpPoint);
end;

function PhysicalToLogicalPointForPerMonitorDPI(hw:HWND; lpPoint:PPoint):BOOL; stdcall;
begin
  result:=_PhysicalToLogicalPointForPerMonitorDPI(hw,lpPoint);
end;

procedure Init;
var
  sv : integer;
  p  : TFarProc;
begin
  Hshcore:=0;
  _GetDPIForMonitor:=FaultGetDPIForMonitor;
  _GetDPIForShellUiComponent:=FaultGetDPIForShellUiComponent;
  _GetProcessDPIAwareness:=FaultGetProcessDPIAwareness;
  _SetProcessDPIAwareness:=FaultSetProcessDPIAwareness;
  _LogicalToPhysicalPointForPerMonitorDPI:=FaultLogicalToPhysicalPointForPerMonitorDPI;
  _PhysicalToLogicalPointForPerMonitorDPI:=FaultPhysicalToLogicalPointForPerMonitorDPI;
  if IsWin81OrLater then begin
    // Windows 8.1以降で有効
    sv:=SetErrorMode(SEM_FAILCRITICALERRORS);
    try
      Hshcore:=LoadLibrary('Shcore.dll');
      if Hshcore<>0 then begin
        p:=GetProcAddress(Hshcore,'GetDpiForMonitor');
        if p<>nil then @_GetDPIForMonitor:=p;
        p:=GetProcAddress(Hshcore,'GetDPIForShellUiComponent');
        if p<>nil then @_GetDPIForShellUiComponent:=p;
        p:=GetProcAddress(Hshcore,'GetProcessDPIAwareness');
        if p<>nil then @_GetProcessDPIAwareness:=p;
        p:=GetProcAddress(Hshcore,'SetProcessDPIAwareness');
        if p<>nil then @_SetProcessDPIAwareness:=p;
        p:=GetProcAddress(Hshcore,'LogicalToPhysicalPointForPerMonitorDPI');
        if p<>nil then @_LogicalToPhysicalPointForPerMonitorDPI:=p;
        p:=GetProcAddress(Hshcore,'PhysicalToLogicalPointForPerMonitorDPI');
        if p<>nil then @_PhysicalToLogicalPointForPerMonitorDPI:=p;
      end;
    finally
      SetErrorMode(sv);
    end;
  end;
end;

procedure Done;
begin
  if Hshcore<>0 then begin
    _GetDPIForMonitor:=FaultGetDPIForMonitor;
    _GetDPIForShellUiComponent:=FaultGetDPIForShellUiComponent;
    _GetProcessDPIAwareness:=FaultGetProcessDPIAwareness;
    _SetProcessDPIAwareness:=FaultSetProcessDPIAwareness;
    _LogicalToPhysicalPointForPerMonitorDPI:=FaultLogicalToPhysicalPointForPerMonitorDPI;
    _PhysicalToLogicalPointForPerMonitorDPI:=FaultPhysicalToLogicalPointForPerMonitorDPI;
    FreeLibrary(Hshcore);
    Hshcore:=0;
  end;
end;

initialization
  Init;

finalization
  Done;

end.
