﻿unit formutils;

{****************************************************************************
 *
 * TForm用各種細かい手続きなど
 *
 * date 2009/07/26
 * auth 綾川鞠乃介
 * memo AppBarの存在も考慮して移動するoptionを追加
 *
 * date 2009/08/11
 * auth 綾川鞠乃介
 * memo 左上優先
 *
 * date 2009/09/25
 * auth 綾川鞠乃介
 * memo multi monitor対応
 *
 * date 2009/12/27
 * auth 綾川鞠乃介
 * memo 位置合わせオプション追加
 *
 * date 2015/08/19
 * auth 綾川鞠乃介
 * memo win8以降のsecondary monitorのタスクバー対応
 *
 * date 2021/09/02
 * auth 綾川鞠乃介
 * memo win10でいつの間にか「タスクバーを自動で隠す」に旨く対応できなくなっていた
 *      のでそれに対応
 *
 ****************************************************************************}

interface

uses
  Winapi.windows,
  System.sysutils,
  Vcl.Forms;

type
  TAdjustPosType = (aptTop,
                    aptBottom,
                    aptLeft,
                    aptRight);
  TAdjustPosTypes = set of TAdjustPosType;

procedure AdjustFormPos(AForm:TForm; AnAppBar:boolean = false; AnAdjusts:TAdjustPosTypes = []);
procedure AdjustFormPosWithMM(AForm:TForm; AnAppBar:boolean = false; AnAdjusts:TAdjustPosTypes = []);

implementation

uses
  getosinf;

procedure AdjustFormPosBase(AForm:TForm; AnAppBar:boolean; AScrRect:TRect; AnAdjusts:TAdjustPosTypes);
var
  rc,workrc : TRect;
  w,h,wl,wt : integer;
  hWnd : THandle;
begin
  with AScrRect do begin
    w:=(right-left+1) div 2; h:=(bottom-top+1) div 2;
  end;
  rc:=AScrRect;
  // ToolBarの存在を加味するならば。
  if AnAppBar then begin
    hWnd:=FindWindow('Shell_TrayWnd',nil); // タスクバーのハンドル
    IF (hwnd<>0) AND GetWindowRect(hWnd,workrc) THEN BEGIN
      // タスクバーの位置を判定
      IF workrc.right<w THEN BEGIN        // タスクバーが左
        if workrc.right>0 then
          rc.Left:=workrc.Right+1;
      END ELSE IF workrc.left>w THEN BEGIN    // タスクバーが右
        if workrc.Left<Screen.Width then
          rc.Right:=workrc.Left-1;
      END ELSE IF workrc.bottom>h THEN BEGIN  // タスクバーが下
        if workrc.Top<Screen.height then
          rc.Bottom:=workrc.Top+1;
      END ELSE BEGIN                      // タスクバーが上
        if workrc.bottom>0 then
          rc.Top:=workrc.Bottom-1;
      END;
    END;
  end;
  // 位置合わせ - 左上優先 - 2009/08/11
  wl:=AForm.Left; wt:=AForm.Top;
  if aptBottom in AnAdjusts then wt:=rc.Bottom-AForm.Height;
  if aptTop in AnAdjusts then wt:=rc.Top;
  if aptRight in AnAdjusts then wl:=rc.Right-AForm.Width;
  if aptLeft in AnAdjusts then wl:=rc.Left;
  if (AForm.Left+AForm.Width)>rc.right then wl:=rc.right-AForm.Width;
  if (AForm.Top+AForm.Height)>rc.bottom then wt:=rc.bottom-AForm.Height;
  if AForm.Left<rc.left then wl:=rc.left;
  if AForm.Top<rc.top  then wt:=rc.top;
  AForm.SetBounds(wl,wt,AForm.Width,AForm.Height);
end;

procedure AdjustFormPos(AForm:TForm; AnAppBar:boolean; AnAdjusts:TAdjustPosTypes);
var
  rc : TRect;
begin
  rc.left:=0; rc.Top:=0; rc.Right:=Screen.Width-1; rc.Bottom:=Screen.Height-1;
  AdjustFormPosBase(AForm,AnAppBar,rc,AnAdjusts);
end;

// 以下、Windows 10以降のタスクバー自動隠蔽対応用
type
  TTrayRects = record
    SrcRect,
    TrayRect : TRect;
    Found : boolean;
  end;
  PTrayRects = ^TTrayRects;

function EnumSTWFunc(_hwnd:HWND; _lParam:LPARAM):BOOL; stdcall;
const
  maxchar = 256;
var
  tr : PTrayRects;
  wr : TRect;
  cn : array[0..maxchar-1] of char;
begin
  if _lparam=0 then
    result:=false
  else begin
    result:=true;
    tr:=PTrayRects(_lParam);
    FillChar(cn,sizeof(cn),0);
    if 0<>GetClassName(_hwnd,@cn[0],maxchar) then begin
      if CompareStr('Shell_SecondaryTrayWnd',string(cn))=0 then begin
        // タスクバーだったのでそれの領域取得
        if GetWindowRect(_hwnd,wr) then begin
          tr.Found:=NOT TRect.Intersect(tr.SrcRect,wr).IsEmpty; // もし空でないなら、それが目的のタスクバー
          if tr.Found then begin
            tr.TrayRect:=wr; // 目的のタスクバーのサイズを格納して終わる
            result:=false;
          end;
        end;
      end;
    end;
  end;
end;
// ここまで

procedure AdjustFormPosWithMM(AForm:TForm; AnAppBar:boolean; AnAdjusts:TAdjustPosTypes);
var
  mn : TMonitor;
  rc : TRect;
  tr : TTrayRects;
  w,h : integer;
  _hwnd : HWND;
begin
  mn:=Screen.MonitorFromWindow(AForm.handle);
  if IsWin10OrLater then begin
    rc:=mn.BoundsRect;
    if AnAppBar then begin
      if mn=Screen.PrimaryMonitor then begin
        _hwnd:=FindWindow('Shell_TrayWnd',nil);
        if _hwnd=0 then
          tr.Found:=false
        else
          tr.Found:=GetWindowRect(_hwnd,tr.TrayRect);
      end else begin
        tr.Found:=false;
        tr.SrcRect:=rc;
        EnumWindows(@EnumSTWFunc,LPARAM(@tr));
      end;
      if tr.Found then begin
        w:=mn.left + mn.Width div 2; h:=mn.top + mn.height div 2;
        if tr.TrayRect.right<w then
          rc.left:=tr.TrayRect.Right
        else if tr.TrayRect.left>w then
          rc.right:=tr.TrayRect.Left
        else if tr.TrayRect.Bottom<h then
          rc.top:=tr.TrayRect.Bottom
        else
          rc.Bottom:=tr.TrayRect.top;
      end;
      AnAppBar:=false;
    end;
  end else if IsWin8OrLater then begin
    if AnAppBar then begin
      rc:=mn.WorkareaRect;
      AnAppBar:=false;
    end else
      rc:=mn.BoundsRect;
  end else begin
    // Win7まではTaskbarはPrimaryのみ
    rc:=mn.BoundsRect;
    AnAppBar:=AnAppBar AND (mn=Screen.PrimaryMonitor);
  end;
  AdjustFormPosBase(AForm,AnAppBar,rc,AnAdjusts);
end;

end.
